import java.awt.*;

/** 
* A Student object for 415.101FC assignment 6
* @author Vincent Wing Hei, Ho
* @version 11.04.2000
*/
public class Student {
    /** Current time */
    private int clock = 0;
    /** Time of needing help for */
    private int helpTime = 0;
    /** Length of help time needed */
    private int helpLength = 0;
    /** Time waited for help since needed */
    private int timeWaiting = 0;
    /** Check if the student is being served */
    private boolean beingServed = false;
    /** X-coordinate for color box around buttons */
    private int xBox;
    /** Y-coordinate for color box around buttons */
    private int yBox;
    /** What student are we dealing at the moment */
    private int studentNumber;
    
    /**
    Creates a new Student.
    Sets up all the necessary values for a new Student.
    @param number the number of the Student
    @param xPos x coordinate of the Button associated with this Student
    @param yPos y coordinate of the Button associated with this Student
    */
    public Student(int number, int xPos, int yPos) {
        studentNumber = number;
        xBox = xPos;
        yBox = yPos;
        helpTime = (int)(Math.random() * 30 + 1) + clock;
        helpLength = (int)(Math.random() * 10 + 1);
    }
    
    /**
    Advances the clock.
    If the student is waiting keeps track of the amount of time waiting.
    If the student is being helped keeps track of how much more time
    the student needs.
    And other things.
    Also prints out the status of this Student.
    */
    public void tick() {
        clock += 1;
        if (clock > helpTime) {
            if (beingServed)        // help is needed, but being served
                if (helpLength >= 1)
                    helpLength -= 1;
                else {              // helpLength reaches 0, generate new helpTime & length
                    timeWaiting = 0;
                    helpTime = (int)(Math.random() * 30 + 1) + clock;
                    helpLength = (int)(Math.random() * 10 + 1);
                }
            else                   // need help, but not being served
                timeWaiting += 1;
        }
        else {
    }
        System.out.println("clock:" + clock + " Student:" + studentNumber + " helpTime:" 
                           + helpTime + " helpLength:" + helpLength + " timeWaiting:" 
                           + timeWaiting + " beingServed:" + beingServed);
    }
    
    
    /**
    Draws the coloured rectangle for the Student.
    Green if the Student is being helped by the demonstrator.
    Various levels of red if the Student is waiting.
    White if the Student is not waiting.
    @param g the Graphics object
    */
    public void draw(Graphics g) {
        int colourCode = 235;
        if (!beingServed) {
            if (clock < helpTime)                // no help is needed
                g.setColor(Color.white);
            else {                               // needed help
                if (clock - helpTime <= 23) {
                    colourCode -= (clock - helpTime) * 10;
                    g.setColor(new Color(255, colourCode, colourCode));
                }
                else
                    g.setColor(new Color(255, 0, 0));
            }
        }
        else {                                   // being helped
            g.setColor(Color.green);
        }
        g.fillRect(xBox - 20, yBox - 20, 100, 60);
    }

    /**
    @return the length of time this Student has been waiting
    */
    public int currentWait() {
        return timeWaiting; 
    }
    
    /**
    Called when the demonstrator arrives to help.
    */
    public void demonstratorArrives() {
        beingServed = true;
    }
    
    /**
    Called when the demonstrator leaves.
    Actually called when the demonstrator arrives at some other Student.
    */
    public void demonstratorLeaves() {
        beingServed = false;
    }
}